@@ -381,7 +381,9 @@ read_file_gdk (Screen *screen, Window window, Drawable drawable,
 
   gdk_pixbuf_xlib_init_with_depth (dpy, screen_number (screen), win_depth);
 # ifdef HAVE_GTK2
+# if !GLIB_CHECK_VERSION(2, 36 ,0)
   g_type_init();
+# endif
 # else  /* !HAVE_GTK2 */
   xlib_rgb_init (dpy, screen);
 # endif /* !HAVE_GTK2 */
@@ -1719,7 +1721,7 @@ get_image (Screen *screen,
         /* Replace slashes with newlines */
         /* while ((dot = strchr(f2, '/'))) *dot = '\n'; */
         /* Replace slashes with spaces */
-        while ((dot = strchr(f2, '/'))) *dot = ' ';
+        /* while ((dot = strchr(f2, '/'))) *dot = ' '; */
 
         XChangeProperty (dpy, window, a, XA_STRING, 8, PropModeReplace, 
                          (unsigned char *) f2, strlen(f2));
